(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./Core/Container", "./Utils/EventDispatcher", "./Core/Utils/Plugins", "./Core/Utils/Constants", "./Utils/NumberUtils", "./Utils/Utils"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.Engine = void 0;
    const Container_1 = require("./Core/Container");
    const EventDispatcher_1 = require("./Utils/EventDispatcher");
    const Plugins_1 = require("./Core/Utils/Plugins");
    const Constants_1 = require("./Core/Utils/Constants");
    const NumberUtils_1 = require("./Utils/NumberUtils");
    const Utils_1 = require("./Utils/Utils");
    async function getDataFromUrl(data) {
        const url = (0, Utils_1.itemFromSingleOrMultiple)(data.url, data.index);
        if (!url) {
            return data.fallback;
        }
        const response = await fetch(url);
        if (response.ok) {
            return response.json();
        }
        console.error(`tsParticles - Error ${response.status} while retrieving config file`);
        return data.fallback;
    }
    class Engine {
        constructor() {
            this._configs = new Map();
            this._domArray = [];
            this._eventDispatcher = new EventDispatcher_1.EventDispatcher();
            this.plugins = new Plugins_1.Plugins(this);
        }
        get configs() {
            const res = {};
            for (const [name, config] of this._configs) {
                res[name] = config;
            }
            return res;
        }
        addConfig(nameOrConfig, config) {
            var _a;
            if (typeof nameOrConfig === "string") {
                if (config) {
                    config.name = nameOrConfig;
                    this._configs.set(nameOrConfig, config);
                }
            }
            else {
                this._configs.set((_a = nameOrConfig.name) !== null && _a !== void 0 ? _a : "default", nameOrConfig);
            }
        }
        addEventListener(type, listener) {
            this._eventDispatcher.addEventListener(type, listener);
        }
        async addInteractor(name, interactorInitializer) {
            this.plugins.addInteractor(name, interactorInitializer);
            await this.refresh();
        }
        async addMover(name, moverInitializer) {
            this.plugins.addParticleMover(name, moverInitializer);
            await this.refresh();
        }
        async addParticleUpdater(name, updaterInitializer) {
            this.plugins.addParticleUpdater(name, updaterInitializer);
            await this.refresh();
        }
        async addPathGenerator(name, generator) {
            this.plugins.addPathGenerator(name, generator);
            await this.refresh();
        }
        async addPlugin(plugin) {
            this.plugins.addPlugin(plugin);
            await this.refresh();
        }
        async addPreset(preset, options, override = false) {
            this.plugins.addPreset(preset, options, override);
            await this.refresh();
        }
        async addShape(shape, drawer, init, afterEffect, destroy) {
            let customDrawer;
            if (typeof drawer === "function") {
                customDrawer = {
                    afterEffect: afterEffect,
                    destroy: destroy,
                    draw: drawer,
                    init: init,
                };
            }
            else {
                customDrawer = drawer;
            }
            this.plugins.addShapeDrawer(shape, customDrawer);
            await this.refresh();
        }
        dispatchEvent(type, args) {
            this._eventDispatcher.dispatchEvent(type, args);
        }
        dom() {
            return this._domArray;
        }
        domItem(index) {
            const dom = this.dom(), item = dom[index];
            if (item && !item.destroyed) {
                return item;
            }
            dom.splice(index, 1);
        }
        async load(params) {
            var _a, _b, _c;
            const id = (_a = params.id) !== null && _a !== void 0 ? _a : `tsparticles${Math.floor((0, NumberUtils_1.getRandom)() * 10000)}`, { index, url } = params, options = url
                ? await getDataFromUrl({
                    fallback: params.options,
                    index,
                    url: url,
                })
                : params.options;
            let domContainer = (_b = params.element) !== null && _b !== void 0 ? _b : document.getElementById(id);
            if (!domContainer) {
                domContainer = document.createElement("div");
                domContainer.id = id;
                (_c = document.querySelector("body")) === null || _c === void 0 ? void 0 : _c.append(domContainer);
            }
            const currentOptions = (0, Utils_1.itemFromSingleOrMultiple)(options, index), dom = this.dom(), oldIndex = dom.findIndex((v) => v.id === id);
            if (oldIndex >= 0) {
                const old = this.domItem(oldIndex);
                if (old && !old.destroyed) {
                    old.destroy();
                    dom.splice(oldIndex, 1);
                }
            }
            let canvasEl;
            if (domContainer.tagName.toLowerCase() === "canvas") {
                canvasEl = domContainer;
                canvasEl.dataset[Constants_1.generatedAttribute] = "false";
            }
            else {
                const existingCanvases = domContainer.getElementsByTagName("canvas");
                if (existingCanvases.length) {
                    canvasEl = existingCanvases[0];
                    canvasEl.dataset[Constants_1.generatedAttribute] = "false";
                }
                else {
                    canvasEl = document.createElement("canvas");
                    canvasEl.dataset[Constants_1.generatedAttribute] = "true";
                    domContainer.appendChild(canvasEl);
                }
            }
            if (!canvasEl.style.width) {
                canvasEl.style.width = "100%";
            }
            if (!canvasEl.style.height) {
                canvasEl.style.height = "100%";
            }
            const newItem = new Container_1.Container(this, id, currentOptions);
            if (oldIndex >= 0) {
                dom.splice(oldIndex, 0, newItem);
            }
            else {
                dom.push(newItem);
            }
            newItem.canvas.loadCanvas(canvasEl);
            await newItem.start();
            return newItem;
        }
        async refresh() {
            for (const instance of this.dom()) {
                await instance.refresh();
            }
        }
        removeEventListener(type, listener) {
            this._eventDispatcher.removeEventListener(type, listener);
        }
        setOnClickHandler(callback) {
            const dom = this.dom();
            if (!dom.length) {
                throw new Error("Can only set click handlers after calling tsParticles.load()");
            }
            for (const domItem of dom) {
                domItem.addClickHandler(callback);
            }
        }
    }
    exports.Engine = Engine;
});
